==> Hacking TCP stack 2.6.12.tar.gz: 2.6.12/tcp_stat.c <==
/*
* Copyright (c) 2007 by ilya (ilya@hackerdom.ru)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; version 2 of the License only.
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

#include <linux/module.h>
#include <linux/kernel.h>

#include <linux/netfilter.h>		// for Netfilter
#include <linux/netfilter_ipv4.h> 	// for some CONSTANT
#include <linux/netdevice.h>		// struct net_device

#include <asm/uaccess.h>
#include <linux/tcp.h>			// tcp_sock
#include <net/tcp.h>			// tcp_ehash_size
#include <linux/inet.h> 		// in_aton

/* Define that we want to get parameters */
MODULE_PARM(sport, "i"); 		// source port
MODULE_PARM(dport, "i"); 		// destination port
MODULE_PARM(saddr, "s"); 		// source address
MODULE_PARM(daddr, "s"); 		// destination address

/* Who am'I */
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Ilya");

/* Define address and port variables */
char *saddr, *daddr;
int sport, dport;

/* Calculate the hash */
int tcp_hashfn (__u32 laddr, __u16 lport, __u32 daddr, __u16 dport)
{

int h = ((laddr ^ lport) ^ (daddr ^ dport));
h ^= h>>16;
h ^= h>>8;

return h & (tcp_ehash_size-1);
}

/* Define netfilter's hook. We will work with incoming packets only. */
struct nf_hook_ops netfilter_ops;

/*
 * Main function!
 * In any case we will return NF_ACCEPT cause we use NetFilter only for analyze
 * tcp protocol.
 */

unsigned int main_hook (unsigned int hooknum, 
			struct sk_buff **skb,
			const struct net_device *in, 
			struct net_device *out, 
			int (*okfn)(struct sk_buff *))
{
struct tcp_sock *tp;
struct sock *sk;		
struct tcp_ehash_bucket *head;
struct sk_buff *sock_buff;
struct tcphdr *th;
struct hlist_node *node;
int hash;

sock_buff=*skb;

/* Check, if this a TCP protocol? */
if (sock_buff->nh.iph->protocol != IPPROTO_TCP)
	return NF_ACCEPT;

th=(struct tcphdr *)(sock_buff->data+20);

/* Is this packet for our connection? */
if (sock_buff->nh.iph->saddr != in_aton(daddr) || sock_buff->nh.iph->daddr != in_aton(saddr)
	|| th->source != htons(dport) || th->dest != htons(sport))
		return NF_ACCEPT;


hash=tcp_hashfn(sock_buff->nh.iph->daddr, ntohs(th->dest), sock_buff->nh.iph->saddr, th->source);
head=&tcp_ehash[hash];

read_lock(&head->lock);
sk_for_each (sk, node, &head->chain){
	if (sk) break;

}
read_unlock(&head->lock);

if (!sk){
	printk ("Have no found any sock...strange thing\n");
	return NF_ACCEPT;
}

tp=tcp_sk(sk);

printk ("Next sequence we send: %u\n", tp->snd_nxt);
return NF_ACCEPT;
}

/* Initialize our module */
int init_module ()
{

	if (!saddr || !sport || !daddr || !dport){
		printk ("Example: insmod tcp_stat.ko daddr=192.168.2.1 saddr=192.168.2.128 sport=1024 dport=139\n");
		printk ("%s, %s, %d, %d\n", daddr, saddr, dport, sport);
		return -EAGAIN;
	}

	/* Fill nf_hook_ops struct */
	netfilter_ops.hook 	= main_hook; 		// Called function
	netfilter_ops.pf	= PF_INET;	
	netfilter_ops.hooknum	= NF_IP_PRE_ROUTING; 	// Where
	netfilter_ops.priority	= NF_IP_PRI_FIRST;	// We should be first!

	/* register our HOOK */
	nf_register_hook(&netfilter_ops);

	printk ("TCP stat is ready to work\n");
	
return 0;
}

/* Get out of here */
void cleanup_module ()
{	
	nf_unregister_hook(&netfilter_ops);
	printk ("TCP stat is unloaded\n");
}

==> Hacking TCP stack 2.6.18.tar.gz: 2.6.18/tcp_stat.c <==
/*
* Copyright (c) 2007 by ilya (ilya@hackerdom.ru)
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; version 2 of the License only.
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
*/

#include <linux/module.h>
#include <linux/kernel.h>

#include <linux/netfilter.h>		// for Netfilter
#include <linux/netfilter_ipv4.h> 	// for some CONSTANT
#include <linux/netdevice.h>		// struct net_device

#include <asm/uaccess.h>
#include <linux/tcp.h>			// tcp_sock
#include <net/tcp.h>			// tcp_death_row
#include <linux/inet.h> 		// in_aton

/* Who am'I */
MODULE_LICENSE("GPL");
MODULE_AUTHOR("Ilya");

/* nodes */
const struct hlist_node *node;

/* Define netfilter's hook. We will work with incoming packets only. */
struct nf_hook_ops netfilter_ops;

/*
 * Main function!
 * In any case we will return NF_ACCEPT cause we use NetFilter only for analyze
 * tcp protocol.
 */

unsigned int main_hook (unsigned int hooknum, 
			struct sk_buff **skb,
			const struct net_device *in, 
			struct net_device *out, 
			int (*okfn)(struct sk_buff *))
{
	
struct sk_buff *sock_buff;
struct inet_ehash_bucket *head;
struct sock *sk=NULL;
struct tcphdr *th;
struct tcp_sock *tp;
unsigned int hash;

sock_buff=*skb;

/* Check, if this a TCP protocol? */
if (sock_buff->nh.iph->protocol != IPPROTO_TCP)
	return NF_ACCEPT;

th=(struct tcphdr *)(sock_buff->data+20);

// How to compute?!
// Local IP (in network order), Local port (host order), Remote IP (network), Remote port (same :)
hash=inet_ehashfn(sock_buff->nh.iph->daddr, ntohs(th->dest), sock_buff->nh.iph->saddr, th->source);
head=inet_ehash_bucket(&tcp_hashinfo, hash);

prefetch(head->chain.first);

write_lock(&head->lock);

sk_for_each (sk, node, &head->chain) {
	
	if (sk) {
		printk ("I've got a socket\n");
		break;
	} 
	
}

write_unlock(&head->lock);

/* have we found any socket? */
if (!sk)
	NF_ACCEPT;

tp=tcp_sk(sk);

printk ("sk->state = %d\n", sk->sk_state);		// Socket state
printk ("sk->protocol = %d\n", sk->sk_protocol);	// Protocol (TCP==6)
printk ("sk->rcvbuf = %d\n", sk->sk_rcvbuf);		// Value of a recv. buffer
printk ("tp->rcv_nxt = %d\n", tp->rcv_nxt); // What we want to receive next


return NF_ACCEPT;
}

/* Initialize our module */
int init_module ()
{
	/* Fill nf_hook_ops struct */
	netfilter_ops.hook 	= main_hook; 		// Called function
	netfilter_ops.pf	= PF_INET;	
	netfilter_ops.hooknum	= NF_IP_PRE_ROUTING; 	// Where
	netfilter_ops.priority	= NF_IP_PRI_FIRST;	// We should be first!

	/* register our HOOK */
	nf_register_hook(&netfilter_ops);

	printk ("TCP stat is ready to work\n");
	
return 0;
}

/* Get out of here */
void cleanup_module ()
{	
	nf_unregister_hook(&netfilter_ops);
	printk ("TCP stat is unloaded\n");
}
